import ccxt
from fundrive import OSSDrive
from fundrive.core.table import DriveTable
from funsecret import read_secret

from funcoin.coins.table.load import LoadTask


def download_daily(days=365, *arge, **kwargs):
    days = int(days)
    exchange = ccxt.binance(  # noqa: F821
        {
            "apiKey": read_secret("coin", "binance", "api_key"),
            "secret": read_secret("coin", "binance", "secret_key"),
        }
    )

    drive = OSSDrive()

    drive.login(
        access_key=read_secret("fundrive", "oss", "farfarfun", "access_key"),
        access_secret=read_secret("fundrive", "oss", "farfarfun", "access_secret"),
        endpoint=read_secret("fundrive", "oss", "farfarfun", "endpoint"),
        bucket_name="farfarfun",
    )

    table = DriveTable(table_fid="funcoin/binance_kline_daily_1m/", drive=drive)
    table.update_partition_meta()
    task = LoadTask(table=table, exchange=exchange)
    task.run(days=days)


# download_daily(days=3)



import json
import os
import time

import requests
from funtool.tool.secret import read_secret
from wechatpy.enterprise import WeChatClient


class HuoBiMessage:
    def __init__(self, name="huobi", webhook=None, agent_id=None, secret=None, company_id=None):
        self.name = name
        # 机器人webhook
        self.webhook = read_secret("wechat", "farfarfun", "huobi", name, "webhook", value=webhook)
        # 应用ID
        self.agent_id = read_secret("wechat", "farfarfun", "huobi", name, "agent_id", value=agent_id)
        # 企业ID
        self.company_id = read_secret("wechat", "farfarfun", "huobi", name, "company_id", value=company_id)
        # 应用Secret
        self.secret = read_secret("wechat", "farfarfun", "huobi", name, "secret", value=secret)
        self.client = WeChatClient(corp_id=self.company_id, secret=self.secret)

    def send_msg(self, msg):
        self.check_token()
        data = {"msgtype": "text", "text": {"content": msg}}
        self.client.message.send(agent_id=self.agent_id, user_ids=["NiuLiangTao", "GuoYe"], party_ids=["3"], msg=data)

        self.send_to_qywx(msg)

    def send_to_qywx(self, msg):
        headers = {"Content-Type": "application/json"}
        data = {"msgtype": "text", "text": {"content": msg}}

        data = json.dumps(data)
        r = requests.post(url=self.webhook, headers=headers, data=data)

    def check_token(self):
        # 通行密钥
        access_token = None
        access_token = read_secret("wechat", "farfarfun", "huobi", self.name, "access_token")
        if not access_token:
            response = self.client.fetch_access_token()
            access_token = read_secret(
                "wechat",
                "farfarfun",
                "huobi",
                self.name,
                "access_token",
                value=response["access_token"],
                expire_time=response["expires_in"],
            )
            print(response)

        self.client.session.set(self.client.access_token_key, access_token)
        return access_token



from .account import AccountAccount
from .base import BaseAPI
from .market import MarketTickers
from .websocket_service import WebsocketService

market_tickers = MarketTickers()
account_account = AccountAccount()
base_api = BaseAPI()
websocket_api = WebsocketService()



import logging

logger = logging.getLogger("funcoin")
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s"))
logger.addHandler(handler)
logger.setLevel(level=logging.INFO)



from time import sleep

from funcoin.huobi.client import GenericClient
from funcoin.huobi.strategy.message import HuoBiMessage
from funcoin.huobi.strategy.trade_order import TradeOrder
from funtool.time import WorkTime, now2unix
from funtool.tool.log import logger
from funtool.tool.secret import read_secret


class FindNewCoin:
    def __init__(self):
        self.unix = 0
        self.msg_huobi = HuoBiMessage()
        self.work_time = WorkTime()

        api_key = read_secret(cate1="coin", cate2="huobi", cate3="api_key")
        secret_key = read_secret(cate1="coin", cate2="huobi", cate3="secret_key")

        self.generic = GenericClient(api_key=api_key, secret_key=secret_key)
        self.symbol_df_last = None
        self.symbol_df_now = None
        self.symbol_dict = {}

    def init(self):
        data = self.generic.get_exchange_symbols().data
        self.symbol_df_last = self.symbol_df_now = data

        for _symbol in data:
            symbol, state = _symbol["symbol"], _symbol["state"]
            self.symbol_dict[symbol] = state

    def update(self, data):
        self.symbol_df_last = self.symbol_df_now
        self.symbol_df_now = data

        for _symbol in data:
            symbol, state = _symbol["symbol"], _symbol["state"]

            if symbol not in self.symbol_dict.keys():
                self.msg_huobi.send_msg(f"new symbol:{symbol}:{state}")
                self.symbol_dict[symbol] = state
                self.new_coin(_symbol)

            elif state != self.symbol_dict[symbol]:
                self.msg_huobi.send_msg(f"state change:{symbol}:{self.symbol_dict[symbol]}->{state}")
                self.symbol_dict[symbol] = state
                self.new_coin(_symbol)
            else:
                pass

    def new_coin(self, _symbol):
        symbol, state = _symbol["symbol"], _symbol["state"]
        if state == "online" and _symbol["quote-currency"] == "usdt" and not symbol.endswith("3lusdt"):
            trade = TradeOrder()
            trade.buy(symbol=_symbol["symbol"], amount=50 * _symbol["min-order-value"])
            self.msg_huobi.send_msg(f"buy new coin:{symbol}:{self.symbol_dict[symbol]}->{state}")

    def run(self):
        step = 1
        self.init()
        while True:
            try:
                data = self.generic.get_exchange_symbols().data
                self.update(data)
                unix = now2unix()
                if unix - self.unix > 86400:
                    self.info()
                    self.unix = unix

                if self.work_time.time_to_day_end(unix):
                    sleep(0.2)
                elif self.work_time.time_to_hour_end(unix):
                    sleep(0.5)
                elif self.work_time.time_to_ten_minute_end(unix):
                    sleep(1)
                else:
                    sleep(5)
                # print(step)
                step += 1
            except Exception as e:
                print(e)
                logger.info(f"error:{e}")

    def info(self):
        status_dict = {}
        for k, v in self.symbol_dict.items():
            ks = status_dict.get(v, [])
            ks.append(k)
            status_dict[v] = ks

        for k, v in status_dict.items():
            self.msg_huobi.send_msg(f"{k}:\t{', '.join(v)}")
            pass


find = FindNewCoin()
find.run()

# ps -elf|grep check_new
# nohup /root/anaconda3/bin/python3.8 /root/workspace/fundata/funcoin/funcoin/huobi/strategy/check_new.py >>/root/workspace/tmp/funcoin-check-run-$(date +%Y-%m-%d).log 2>&1 &



from funcoin.huobi.client import AccountClient, GenericClient, MarketClient, TradeClient
from funcoin.huobi.constant import OrderType
from funtool.tool.secret import read_secret


class TradeOrder:
    def __init__(self):
        self.trade: TradeClient = None
        self.market: MarketClient = None
        self.generic: GenericClient = None
        self.account: AccountClient = None
        self.account_id = None

        self.init()

    def init(self):
        api_key = read_secret(cate1="coin", cate2="huobi", cate3="api_key")
        secret_key = read_secret(cate1="coin", cate2="huobi", cate3="secret_key")
        self.market = MarketClient(api_key=api_key, secret_key=secret_key)
        self.generic = GenericClient(api_key=api_key, secret_key=secret_key)
        self.trade = TradeClient(api_key=api_key, secret_key=secret_key)

        self.account = AccountClient(api_key=api_key, secret_key=secret_key)

        account_dict = self.account.get_accounts()
        # print(account_dict.dict_data['data'][0]['id'])
        self.account_id = account_dict.dict_data["data"][0]["id"]

    def buy(self, symbol, amount):
        source = "spot-api"
        return self.trade.create_order(
            symbol=symbol,
            account_id=self.account_id,
            order_type=OrderType.BUY_MARKET,
            amount=amount,
            source=source,
            price=0,
        )



import json
import time

from funcoin.huobi.connection import (RestApiSyncClient, SubscribeClient,
                                       WebSocketReqClient)
from funcoin.huobi.constant import (CandlestickInterval, DepthStep,
                                     HttpMethod, MbpLevel)
from funcoin.huobi.utils import get_current_timestamp
from funcoin.huobi.utils.input_checker import (check_in_list, check_range,
                                                check_should_not_none,
                                                check_symbol,
                                                check_symbol_list)
from funcoin.huobi.utils.model import Response


class MarketClient(object):

    def __init__(self, *args, **kwargs):
        """
        Create the request client instance.
        :param kwargs: The option of request connection.
            api_key: The public key applied from Huobi.
            secret_key: The private key applied from Huobi.
            url: The URL name like "https://api.huobi.pro".
            init_log: to init logger
        """
        self.__kwargs = kwargs
        self.market_service = RestApiSyncClient(*args, **kwargs)
        self.market_service_sub = SubscribeClient(**kwargs)
        self.market_service_socket = WebSocketReqClient(**kwargs)

    def get_candlestick(self, symbol, period, size=200):
        """
        Get the candlestick/kline for the specified symbol. The data number is 150 as default.

        :param symbol: The symbol, like "btcusdt". To query hb10, put "hb10" at here. (mandatory)
        :param period: The candlestick/kline interval, MIN1, MIN5, DAY1 etc. (mandatory)
        :param size: The start time of of requested candlestick/kline data. (optional)
        :return: The list of candlestick/kline data.
        """
        check_symbol(symbol)
        check_should_not_none(period, "period")
        check_range(size, 1, 2000, "size")

        params = {
            "symbol": symbol,
            "period": period,
            "size": size
        }
        channel = "/market/history/kline"
        return self.market_service.request_process(HttpMethod.GET, channel, params)

    def sub_candlestick(self, symbols: str, interval: CandlestickInterval, callback, error_handler):
        """
        Subscribe candlestick/kline event. If the candlestick/kline is updated,
        server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param interval: The candlestick/kline interval, MIN1, MIN5, DAY1 etc.
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(candlestick_event: 'CandlestickEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen
        between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return: No return
        """

        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(interval, "interval")
        check_should_not_none(callback, "callback")

        def kline_channel(symbol, _interval):
            channel = dict()
            channel["sub"] = "market." + symbol + ".kline." + _interval
            channel["id"] = str(get_current_timestamp())
            return json.dumps(channel)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(kline_channel(symbol, interval))
                time.sleep(0.01)

        self.market_service_sub.execute_subscribe_v1(subscription, callback, error_handler)

    def req_candlestick(self, symbols: str, interval: CandlestickInterval, callback,
                        from_ts_second=None, end_ts_second=None, error_handler=None):
        """
        Subscribe candlestick/kline event. If the candlestick/kline is updated,
        server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param interval: The candlestick/kline interval, MIN1, MIN5, DAY1 etc.
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(candlestick_event: 'CandlestickEvent'):
                        pass
        :param from_ts_second : data from timestamp [it's second]
        :param end_ts_second : data util timestamp [it's second]
        :param error_handler: The error handler will be called if subscription failed or error happen
        between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return: No return
        """

        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(interval, "interval")
        check_should_not_none(callback, "callback")

        def request_kline_channel(symbol, _interval, _from_ts_second=None, to_ts_second=None):
            channel = dict()
            channel["req"] = "market." + symbol + ".kline." + _interval
            channel["id"] = str(get_current_timestamp())
            if _from_ts_second:
                channel["from"] = int(_from_ts_second)
            if to_ts_second:
                channel["to"] = int(to_ts_second)
            return json.dumps(channel)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(request_kline_channel(symbol, interval, from_ts_second, end_ts_second))
                time.sleep(0.01)

        self.market_service_socket.execute_subscribe_v1(subscription, callback, error_handler)

    def get_price_depth(self, symbol: str, depth_type: str, depth_size: int = None):
        """
        Get the Market Depth of a symbol.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :param depth_type: The tpye, like "step0" to "step5". (mandatory)
        :param depth_size: (optional) The maximum number of Market Depth step0 requested. range [1 - 150],
                           default is 150
                           The maximum number of Market Depth step1,step2,step3,step4,step5 requested.
                           size is in [5, 10, 20], default is 20.
        :return: Market Depth data.
        """
        channel = "/market/depth"

        check_symbol(symbol)
        check_in_list(depth_type, [DepthStep.STEP0, DepthStep.STEP1, DepthStep.STEP2, DepthStep.STEP3, DepthStep.STEP4,
                                   DepthStep.STEP5], "depth_type")
        params = {
            "symbol": symbol,
            "type": depth_type,
            # "depth": depth_size
        }

        ret_data = self.market_service.request_process(HttpMethod.GET, channel, params)

        if depth_size is not None:
            if (ret_data.bids is not None) and (len(ret_data.bids) > depth_size):
                ret_data.bids = ret_data.bids[0:depth_size]

            if (ret_data.asks is not None) and (len(ret_data.asks) > depth_size):
                ret_data.asks = ret_data.asks[0:depth_size]

        return ret_data

    @staticmethod
    def get_depth_step_list():
        return [DepthStep.STEP0, DepthStep.STEP1, DepthStep.STEP2, DepthStep.STEP3, DepthStep.STEP4, DepthStep.STEP5]

    @staticmethod
    def get_valid_depth_step(value, defalut_value):
        step_list = MarketClient.get_depth_step_list()
        if value in step_list:
            return value
        else:
            return defalut_value

    def sub_price_depth(self, symbols: str, depth_step: str, callback, error_handler=None):
        """
        Subscribe price depth event. If the price depth is updated,
        server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param depth_step: The depth precision, string from step0 to step5.
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen
                              between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        step = MarketClient.get_valid_depth_step(value=depth_step, defalut_value=DepthStep.STEP0)
        check_should_not_none(callback, "callback")

        def price_depth_channel(symbol, step_type=DepthStep.STEP0):
            channel = dict()
            channel["sub"] = "market." + symbol + ".depth." + step_type
            channel["id"] = str(get_current_timestamp())
            return json.dumps(channel)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(price_depth_channel(symbol, step))
                time.sleep(0.01)

        self.market_service_sub.execute_subscribe_v1(subscription, callback, error_handler)

    def sub_price_depth_bbo(self, symbols: str, callback, error_handler=None):
        """
        Subscribe price depth event. If the price depth is updated,
        server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen
                              between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(callback, "callback")

        def price_depth_bbo_channel(symbol):
            channel = dict()
            channel["sub"] = "market." + symbol + ".bbo"
            channel["id"] = str(get_current_timestamp())
            return json.dumps(channel)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(price_depth_bbo_channel(symbol))
                time.sleep(0.01)

        self.market_service_sub.execute_subscribe_v1(subscription, callback, error_handler)

    def req_price_depth(self, symbols: str, depth_step: str, callback, error_handler=None):
        """
        Subscribe price depth event. If the price depth is updated,
        server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param depth_step: The depth precision, string from step0 to step5.
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen
                              between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        step = MarketClient.get_valid_depth_step(value=depth_step, defalut_value=DepthStep.STEP0)
        check_should_not_none(callback, "callback")

        def request_price_depth_channel(symbol, step_type="step0"):
            channel = dict()
            channel["req"] = "market." + symbol + ".depth." + step_type
            channel["id"] = str(get_current_timestamp())
            return json.dumps(channel)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(request_price_depth_channel(symbol, step))
                time.sleep(0.01)

        self.market_service_socket.execute_subscribe_v1(subscription, callback, error_handler)

    def get_market_detail(self, symbol: str):
        """
        Get trade statistics in 24 hours.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :return: Trade statistics.
        """
        channel = "/market/detail"

        check_symbol(symbol)

        params = {
            "symbol": symbol,
        }

        return self.market_service.request_process(HttpMethod.GET, channel, params)

    def sub_market_detail(self, symbols: str, callback, error_handler=None):
        """
        Subscribe 24 hours trade statistics event. If statistics is generated,
        server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(trade_statistics_event: 'TradeStatisticsEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen
                              between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(callback, "callback")

        def market_detail_channel(_symbol):
            channel = dict()
            channel["sub"] = "market." + _symbol + ".detail"
            channel["id"] = str(get_current_timestamp())
            return json.dumps(channel)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(market_detail_channel(symbol))
                time.sleep(0.01)

        self.market_service_sub.execute_subscribe_v1(subscription, callback, error_handler)

    def req_market_detail(self, symbols: str, callback, error_handler=None):
        """
        Subscribe 24 hours trade statistics event. If statistics is generated,
        server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(trade_statistics_event: 'TradeStatisticsEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen
        between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(callback, "callback")

        def request_market_detail_channel(_symbol):
            channel = dict()
            channel["req"] = "market." + _symbol + ".detail"
            channel["id"] = str(get_current_timestamp())
            return json.dumps(channel)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(request_market_detail_channel(symbol))
                time.sleep(0.01)

        self.market_service_socket.execute_subscribe_v1(subscription, callback, error_handler)

    def get_market_trade(self, symbol: str) -> list:
        """
        Get the most recent trades with their price, volume and direction.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :return: The list of trade.
        """
        channel = "/market/trade"

        check_symbol(symbol)

        params = {
            "symbol": symbol,
        }

        return self.market_service.request_process(HttpMethod.GET, channel, params)

    def get_history_trade(self, symbol: str, size: 'int' = None) -> list:
        """
        Get the most recent trades with their price, volume and direction.

        :param symbol: The symbol, like "btcusdt". (mandatory)
        :param size: The number of historical trade requested, range [1 - 2000] (optional)
        :return: The list of trade.
        """
        channel = "/market/history/trade"

        check_symbol(symbol)
        check_range(size, 1, 2000, "size")

        params = {
            "symbol": symbol,
            "size": size
        }

        return self.market_service.request_process(HttpMethod.GET, channel, params)

    def sub_trade_detail(self, symbols: str, callback, error_handler=None):
        """
        Subscribe price depth event. If the price depth is updated,
        server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(trade_event: 'TradeEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error
        happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(callback, "callback")

        def trade_detail_channel(symbol):
            channel = dict()
            channel["sub"] = "market." + symbol + ".trade.detail"
            channel["id"] = str(get_current_timestamp())
            return json.dumps(channel)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(trade_detail_channel(symbol))
                time.sleep(0.01)

        self.market_service_sub.execute_subscribe_v1(subscription, callback, error_handler)

    def req_trade_detail(self, symbols: str, callback, error_handler=None):
        """
        Subscribe price depth event. If the price depth is updated,
        server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(trade_event: 'TradeEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error
        happen between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass
        :return:  No return
        """
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(callback, "callback")

        def request_trade_detail_channel(symbol):
            channel = dict()
            channel["req"] = "market." + symbol + ".trade.detail"
            channel["id"] = str(get_current_timestamp())
            return json.dumps(channel)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(request_trade_detail_channel(symbol))
                time.sleep(0.01)

        self.market_service_socket.execute_subscribe_v1(subscription, callback, error_handler)

    def get_market_detail_merged(self, symbol):
        check_symbol(symbol)
        params = {
            "symbol": symbol
        }
        channel = "/market/detail/merged"
        return self.market_service.request_process(HttpMethod.GET, channel, params)

    def get_market_tickers(self) -> Response:
        """
        get market tickers

        :return: market ticker list.
        """

        params = {}
        channel = "/market/tickers"
        return self.market_service.request_process(HttpMethod.GET, channel, params)

    """
    increase mbp(market by price)
    """

    def sub_mbp_increase(self, symbols: str, levels: 'int', callback, error_handler=None):
        """
        Subscribe mbp event. If the mbp is updated,
        server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param levels: level, 5，10，20，150. current only support 150
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen
        between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """
        check_should_not_none(symbols, "symbol")
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(levels, "levels")
        check_should_not_none(callback, "callback")

        def mbp_increase_channel(symbol, level):
            channel = dict()
            channel["sub"] = "market.{symbol}.mbp.{level}".format(symbol=symbol, level=level)
            channel["id"] = str(get_current_timestamp())
            return json.dumps(channel)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(mbp_increase_channel(symbol, levels))
                time.sleep(0.01)

        return self.market_service_sub.execute_subscribe_mbp(subscription, callback, error_handler)

    """
    subscribe full mbp(market by price)
    """

    def sub_mbp_full(self, symbols: str, levels: 'int', callback, error_handler=None):
        """
        Subscribe full mbp event. If the mbp is updated,
        server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param levels: level, 5，10，20
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param error_handler: The error handler will be called if subscription failed or error happen
        between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """
        check_should_not_none(symbols, "symbol")
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(levels, "levels")
        check_in_list(levels, [MbpLevel.MBP5, MbpLevel.MBP10, MbpLevel.MBP20], "levels")
        check_should_not_none(callback, "callback")

        def mbp_full_channel(symbol, level):
            channel = dict()
            channel["sub"] = "market.{symbol}.mbp.refresh.{level}".format(symbol=symbol, level=level)
            channel["id"] = str(get_current_timestamp())
            return json.dumps(channel)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(mbp_full_channel(symbol, levels))
                time.sleep(0.01)

        self.market_service_sub.execute_subscribe_v1(subscription, callback, error_handler)

    def req_mbp(self, symbols: str, levels: 'int', callback, auto_close=True, error_handler=None):
        """
        Subscribe mbp event. If the mbp is updated,
        server will send the data to client and onReceive in callback will be called.

        :param symbols: The symbols, like "btcusdt". Use comma to separate multi symbols, like "btcusdt,ethusdt".
        :param levels: level, 5，10，20，150. current only support 150
        :param callback: The implementation is required. onReceive will be called if receive server's update.
            example: def callback(price_depth_event: 'PriceDepthEvent'):
                        pass
        :param auto_close : close websocket connection after get data
        :param error_handler: The error handler will be called if subscription failed or error happen
        between client and Huobi server
            example: def error_handler(exception: 'HuobiApiException')
                        pass

        :return:  No return
        """
        check_should_not_none(symbols, "symbol")
        symbol_list = symbols.split(",")
        check_symbol_list(symbol_list)
        check_should_not_none(levels, "levels")
        check_should_not_none(callback, "callback")

        def request_mbp_channel(symbol, level):
            channel = dict()
            channel["req"] = "market.{symbol}.mbp.{level}".format(symbol=symbol, level=level)
            channel["id"] = str(get_current_timestamp())
            return json.dumps(channel)

        def subscription(connection):
            for symbol in symbol_list:
                connection.send(request_mbp_channel(symbol, levels))
                time.sleep(0.01)

        self.market_service_socket.execute_subscribe_mbp(subscription, callback, error_handler)



import json

import pandas as pd
from funcoin.huobi.utils.etf_result import etf_result_check
from funcoin.huobi.utils.huobi_api_exception import HuobiApiException
from funcoin.huobi.utils.print_mix_object import TypeCheck


def check_response(dict_data):
    status = dict_data.get("status", None)
    code = dict_data.get("code", None)
    success = dict_data.get("success", None)
    if status and len(status):
        if TypeCheck.is_basic(status):  # for normal case
            if status == "error":
                err_code = dict_data.get("err-code", 0)
                err_msg = dict_data.get("err-msg", "")
                raise HuobiApiException(HuobiApiException.EXEC_ERROR,
                                        "[Executing] " + str(err_code) + ": " + err_msg)
            elif status != "ok":
                raise HuobiApiException(HuobiApiException.RUNTIME_ERROR,
                                        "[Invoking] Response is not expected: " + status)
        # for https://status.huobigroup.com/api/v2/summary.json in example example/generic/get_system_status.py
        elif TypeCheck.is_dict(status):
            if dict_data.get("page") and dict_data.get("components"):
                pass
            else:
                raise HuobiApiException(HuobiApiException.EXEC_ERROR, "[Executing] System is in maintenances")
    elif code:
        code_int = int(code)
        if code_int != 200:
            err_code = dict_data.get("code", 0)
            err_msg = dict_data.get("message", "")
            raise HuobiApiException(HuobiApiException.EXEC_ERROR,
                                    "[Executing] " + str(err_code) + ": " + err_msg)
    elif success is not None:
        if bool(success) is False:
            err_code = etf_result_check(dict_data.get("code"))
            err_msg = dict_data.get("message", "")
            if err_code == "":
                raise HuobiApiException(HuobiApiException.EXEC_ERROR, "[Executing] " + err_msg)
            else:
                raise HuobiApiException(HuobiApiException.EXEC_ERROR, "[Executing] " + str(err_code) + ": " + err_msg)
    else:
        raise HuobiApiException(HuobiApiException.RUNTIME_ERROR, "[Invoking] Status cannot be found in response.")


class Response:
    def __init__(self, dict_data=None, response=None, code=None, data=None):
        if response is not None:
            try:
                dict_data = json.loads(response, encoding="utf-8")
                check_response(dict_data)
            except Exception as e:
                print(f"{e}:{response}")
        

        dict_data = dict_data or {}
        self.dict_data = dict_data

        self.code = dict_data.get('code', -1)
        self.data = dict_data.get('data', [])
        if self.data is not None and len(self.data) > 0:
            try:
                self.data_df = pd.DataFrame(self.data)
            except Exception as e:
                print(f"{e}:{response}")

    def __str__(self):
        return self.code



from funbuild.shell import run_shell_list
from funcoin.coins.task.download import download_daily
from funserver.base import BaseServer, server_parser


class FunCoinDownload(BaseServer):
    def __init__(self):
        super().__init__(server_name="funcoin-download")

    def update(self, args=None, **kwargs):
        run_shell_list(["pip install funcoin -U"])

    def run(self, *args, **kwargs):
        download_daily()


def funcoin_download():
    server = FunCoinDownload()
    parser, subparsers = server_parser(server)

    args = parser.parse_args()
    params = vars(args)
    args.func(**params)



from funbuild.shell import run_shell_list
from funcoin.coins.task.download import download_daily
from funserver.base import BaseServer, server_parser


class FunCoin(BaseServer):
    def __init__(self):
        super().__init__(server_name="funcoin")

    def update(self, args=None, **kwargs):
        run_shell_list(["pip install funcoin -U"])

    def run(self, *args, **kwargs):
        pass


def funcoin():
    server = FunCoin()
    parser, subparsers = server_parser(server)

    build_parser1 = subparsers.add_parser("download", help="download daily")
    build_parser1.add_argument("--days", default=365, help="days")
    build_parser1.set_defaults(func=download_daily)

    args = parser.parse_args()
    params = vars(args)
    args.func(**params)



import logging

from fastapi import APIRouter
from darkbuild.tool.fastapi import add_api_routes, api_route
from funcoin.database.base import create_all, create_session
from funcoin.database.connect.base import BaseConnect
from funcoin.okex.database.client import OkexClientAccountBalance
from funcoin.okex.database.websocket import OkexWebsocketChannels
from funcoin.okex.websocket.channel import PublicChannel
from funcoin.okex.websocket.connect import PublicConnect
from funcoin.okex.websocket.handle import PublicTickers, ResponseHandel


class WebsocketService(APIRouter):
    def __init__(self, connect: BaseConnect = None, prefix='/websocket', *args, **kwargs):
        self.connect = connect or PublicConnect(channels=[PublicChannel.public_tickers().to_json()])
        self.connect.add_handle(PublicTickers())
        self.connect.add_handle(ResponseHandel())
        self.session = create_session()

        super(WebsocketService, self).__init__(prefix=prefix, *args, **kwargs)
        add_api_routes(self)

        self.connect.run()

    def update_channels_db(self):
        details = self.session.query(OkexClientAccountBalance).filter(OkexClientAccountBalance.eqUsd > 10).all()

        for _detail in details:
            detail = _detail.json()
            if detail['ccy'] == 'USDT':
                continue
            instId = f"{detail['ccy']}-USDT"
            param = {
                "channel_json": str(PublicChannel.public_tickers(instId).to_json()),
                "channel": "tickers",
                "instId": instId
            }
            self.session.merge(OkexWebsocketChannels(**param))
        create_all()
        self.session.commit()
        return {"update_db": len(details)}

    @api_route('/update/channel', description="get value")
    def update_channels(self):
        try:
            res = self.update_channels_db()

            new_channels = self.session.query(OkexWebsocketChannels).all()
            inst_list = [channel.instId for channel in new_channels]
            new_channels = [eval(channel.channel_json) for channel in new_channels]

            new_channels_str = [str(channel) for channel in new_channels]
            old_channels_str = [str(channel) for channel in self.connect.channels]

            if new_channels_str is not None and len(new_channels_str) > 0:
                if len(list(set(old_channels_str) - set(new_channels_str))) > 0 or len(
                        list(set(new_channels_str) - set(old_channels_str))) > 0:
                    logging.info(f"old:{old_channels_str},new:{new_channels_str}")
                    self.connect.channels = new_channels
                    self.connect.subscribe_restart()
                    res['subscribe'] = 'restart'

            res["channels_size"] = len(self.connect.channels)
            res["update"] = "success"
            res['inst_list'] = inst_list
        except Exception as e:
            self.session.rollback()
            res["update"] = "failed"
            res["error"] = str(e)
        return res



from funtool.secret import read_secret

from .account import AccountClient
from .asset import AssetClient
from .market import MarketClient
from .public import PublicClient
from .trade import TradeClient

api_key = read_secret(cate1="coin", cate2="okex", cate3="api_key")
secret_key = read_secret(cate1="coin", cate2="okex", cate3="secret_key")
passphrase = read_secret(cate1="coin", cate2="okex", cate3="passphrase")

account_api = AccountClient(api_key, secret_key, passphrase)
funding_api = AssetClient(api_key, secret_key, passphrase)
market_api = MarketClient(api_key, secret_key, passphrase)
public_api = PublicClient(api_key, secret_key, passphrase)
trade_api = TradeClient(api_key, secret_key, passphrase)
